@@ -20,13 +20,9 @@
 #
 ###############################################################################
 
-############### IMPORTS #######################################################
-
 from PySide6.QtWidgets import QApplication
 from PySide6.QtCore    import QTimer
 
-# import atexit
-
 ############### QT APPLICATION ################################################
 # The QApplication object manages PyQt Widgets and their events.
 # The .exec() method builds the application, starts the event loop,
@@ -120,6 +116,8 @@ class Timer():
       # create an internal QTimer
       self._timer = QTimer()
 
+      self.parameters = parameters
+
       self.setDelay(timeInterval)               # how often should the timer trigger?
       self.setRepeat(repeat)                    # should we do this once or forever?
       self.setFunction(function, parameters)    # register callback function
@@ -130,11 +128,11 @@ class Timer():
 
    def __str__(self):
       """Return a string representation of the timer."""
-      return f"Timer: {self.getDelay()} ms, repeat: {self.getRepeat()}"   # string representation of the timer
+      return f"Timer(timeInterval = {self.getDelay()}, function = {self._function}, parameters = {self.parameters}, repeat = {self.getRepeat()})"
 
    def __repr__(self):
       """Return the string representation for repr."""
-      return str(self)   # use the string representation for repr
+      return str(self)
 
    def _run(self):
       """Call the user-supplied function with parameters."""
@@ -178,6 +176,22 @@ class Timer():
       repeat
       self._timer.setSingleShot(not repeat)   # set the timer to single shot if repeat is false
 
+
+############### Timer2 #################################################################
+# To work around the limitation that we can't easily schedule callbacks at specific times,
+# we create a ticker thread that wakes up at short intervals to check if any timers
+# need to be triggered.
+#
+# ACCURACY ADVANTAGES:
+# 1. Running in a separate thread means timing isn't affected by operations in the main thread
+# 2. Measuring actual elapsed time (dt) compensates for any processing delays
+# 3. Accumulating precise time values prevents long-term drift
+#
+# Timers are added to _activeTimers when they start, and removed when they stop
+# or, if they are oneshots, when they call their callback function.
+
+import threading, time, atexit
+
 class Timer2:
    """Custom timer used to schedule tasks to be run at fixed time intervals."""
 
@@ -193,7 +207,7 @@ class Timer2:
       self._running = False                    # flag indicating if timer is currently running
 
    def __str__(self):
-      return f"Timer: {self.getDelay()} ms, repeat: {self.getRepeat()}"
+      return f"Timer(timeInterval = {self.getDelay()}, function = {self._function}, parameters = {self._parameters}, repeat = {self.getRepeat()})"
 
    def __repr__(self):
       return str(self)
@@ -226,7 +240,7 @@ class Timer2:
       if not self._running:
          self._running = True         # set running flag to true
          self._accumulatedTime = 0    # reset accumulated time
-         # _timers.append(self)   # add this timer to the active timers list
+         _activeTimers.append(self)   # add this timer to the active timers list
 
    def stop(self):
       """Stops scheduled task from executing."""
@@ -234,9 +248,9 @@ class Timer2:
       if self._running:
          self._running = False         # set running flag to false
 
-         # # remove this timer from the active timers list if it exists
-         # if self in _timers:
-         #    _timers.remove(self)
+         # remove this timer from the active timers list if it exists
+         if self in _activeTimers:
+            _activeTimers.remove(self)
 
    def getDelay(self):
       """Returns the delay time interval (in milliseconds)."""
@@ -271,6 +285,65 @@ class Timer2:
       """Timer is set to repeat if flag is True, and not to repeat if flag is False."""
       self._repeat = repeat
 
+
+# global ticker thread that calls timer callbacks
+def _tickerThread():
+   """
+   Internal function that periodically updates all active timers.
+
+   This function runs in a separate thread and wakes up at regular intervals
+   to call the _tick method of all active timers. It measures the actual time
+   elapsed between calls to ensure timing accuracy.
+   """
+
+   lastTime = time.time()
+   while _tickerRunning:
+      currentTime = time.time()
+      dt = currentTime - lastTime  # measure actual elapsed time between ticks
+      lastTime = currentTime
+
+      # call tick for all active timers with the precise time difference
+      for timer in list(_activeTimers):
+         timer._tick(dt)
+
+      # sleep for a short time (adjust as needed for precision)
+      time.sleep(0.01)  # 10ms resolution - balances CPU usage with timing precision
+
+
+# register cleanup on exit
+def _cleanup():
+   """Cleans up all active timers when the program exits.
+
+   This function is registered with atexit to ensure proper cleanup
+   even if the program terminates unexpectedly.
+   """
+   global _tickerRunning
+   _tickerRunning = False
+
+   # stop all active timers
+   for timer in list(_activeTimers):
+      timer.stop()
+
+
+# if this is the first time loading this module
+_initialized = False
+if not _initialized:
+   _initialized = True
+   _activeTimers = []
+   _tickerRunning = True
+
+   # allow Python to exit even if thread is running
+   _ticker = threading.Thread(target=_tickerThread, daemon=True)
+
+   # start ticker thread
+   _ticker.start()
+
+   # NOTE: We don't need a blocking thread when using python -i,
+   # since the interpreter stays open already
+
+   atexit.register(_cleanup)   # cleanup active timers at exit
+
+
 #######################################################################################
 # LinearRamp
 #
@@ -385,166 +458,8 @@ class LinearRamp:
             self._timer.stop()
             self._timer = None
 
-############### Timer2 #################################################################
-# To work around the limitation that we can't easily schedule callbacks at specific times,
-# we create a ticker thread that wakes up at short intervals to check if any timers
-# need to be triggered.
-#
-# ACCURACY ADVANTAGES:
-# 1. Running in a separate thread means timing isn't affected by operations in the main thread
-# 2. Measuring actual elapsed time (dt) compensates for any processing delays
-# 3. Accumulating precise time values prevents long-term drift
-#
-# Timers are added to _ACTIVE_TIMERS when they start, and removed when they stop
-# or, if they are oneshots, when they call their callback function.
-
-# global ticker thread that calls timer callbacks
-
-import threading, time, atexit
-
-def _tickerThread():
-   """Internal function that periodically updates all active timers.
-
-   This function runs in a separate thread and wakes up at regular intervals
-   to call the _tick method of all active timers. It measures the actual time
-   elapsed between calls to ensure timing accuracy.
-   """
-   lastTime = time.time()
-   while _tickerRunning:
-      currentTime = time.time()
-      dt = currentTime - lastTime  # measure actual elapsed time between ticks
-      lastTime = currentTime
-
-      # call tick for all active timers with the precise time difference
-      for timer in list(_activeTimers):
-         timer._tick(dt)
-
-      # sleep for a short time (adjust as needed for precision)
-      time.sleep(0.01)  # 10ms resolution - balances CPU usage with timing precision
-
-_initialized = False
-# if this is the first time loading this module, initialize global variables
-if not _initialized:
-   _initialized = True
-   _activeTimers = []
-   _tickerRunning = True
-
-   # allow Python to exit even if thread is running
-   _ticker = threading.Thread(target=_tickerThread, daemon=True)
-   # start ticker thread
-   _ticker.start()
-
-   # Note: We don't need a blocking thread when using python -i,
-   # since the interpreter stays open already
-
-   # register cleanup on exit
-   def _cleanup():
-      """Cleans up all active timers when the program exits.
-
-      This function is registered with atexit to ensure proper cleanup
-      even if the program terminates unexpectedly.
-      """
-      global _tickerRunning
-      _tickerRunning = False
-
-      # stop all active timers
-      for timer in list(_activeTimers):
-         timer.stop()
-
-   atexit.register(_cleanup)
-
-class Timer2:
-   """Timer used to schedule tasks to be run at fixed time intervals."""
 
 
-   def __init__(self, timeInterval, function, parameters=[], repeat=True):
-      """Specify time interval (in milliseconds), which function to call when the time interval has passed
-         and the parameters to pass this function, and whether to repeat (True) or do it only once."""
-      self._interval = timeInterval / 1000.0   # convert ms to seconds to work with Python's Time module
-      self._function = function                # callback function to execute
-      self._parameters = parameters            # parameters to pass to the callback function
-      self._repeat = repeat                    # whether to repeat the timer or run once
-      self._scheduled = False                  # flag indicating if timer is scheduled
-      self._accumulatedTime = 0                # tracks elapsed time since last execution
-      self._running = False                    # flag indicating if timer is currently running
-
-   def __str__(self):
-      return f"Timer: {self.getDelay()} ms, repeat: {self.getRepeat()}"
-
-   def __repr__(self):
-      return str(self)
-
-   def _tick(self, dt):
-      """Internal method called by the ticker thread to update timer state."""
-      if not self._running:
-         return
-
-      # accumulate the actual elapsed time with high precision
-      self._accumulatedTime += dt
-
-      # check if it's time to run the callback
-      if self._accumulatedTime >= self._interval:
-         # call the function
-         self._function(*self._parameters)
-
-         # reset accumulated time, accounting for overshooting
-         if self._repeat:
-            # DRIFT PREVENTION: subtract exactly one interval, preserving any excess time
-            # this is critical for preventing long-term drift in repeated timers
-            self._accumulatedTime -= self._interval
-         else:
-            # for one-shot timers, remove from active timers
-            self.stop()
-
-   def start(self):
-      """Creates a timer task to perform the desired task as specified (in terms of timeInterval and repeat)."""
-      # check if timer is not already running
-      if not self._running:
-         self._running = True         # set running flag to true
-         self._accumulatedTime = 0    # reset accumulated time
-         _activeTimers.append(self)   # add this timer to the active timers list
-
-   def stop(self):
-      """Stops scheduled task from executing."""
-      # check if timer is currently running
-      if self._running:
-         self._running = False         # set running flag to false
-         # remove this timer from the active timers list if it exists
-         if self in _activeTimers:
-            _activeTimers.remove(self)
-
-   def getDelay(self):
-      """Returns the delay time interval (in milliseconds)."""
-      return self._interval * 1000   # convert back to ms for API consistency
-
-   def setDelay(self, timeInterval):
-      """
-      Sets a new delay time interval for timer t (in milliseconds).
-         This allows to change the speed of the animation, after some event occurs..
-      """
-      self._interval = timeInterval / 1000.0
-
-      # if running, restart with new interval
-      if self.isRunning():
-         self.stop()
-         self.start()
-
-   def isRunning(self):
-      """Returns True if timer is still running, False otherwise."""
-      return self._running
-
-   def setFunction(self, function, parameters=[]):
-      """Sets the function to execute.  The optional parameter parameters is a list of parameters to pass to the function (when called)."""
-      self._function = function
-      self._parameters = parameters
-
-   def getRepeat(self):
-      """Returns True if timer is set to repeat, False otherwise."""
-      return self._repeat
-
-   def setRepeat(self, repeat):
-      """Timer is set to repeat if flag is True, and not to repeat if flag is False."""
-      self._repeat = repeat
 
 # Tests
 
